import { Hono, Context } from 'hono';
import { PostPage } from '@blog/templates/Pages/PostPage';
import { FourOhFour } from '@blog/templates/Pages/FourOhFour';
import { SiteMeta } from '@blog/models/SiteMeta';
import { PostService } from '@blog/services/post-file';

type Posts = {
  postService: PostService 
}

const posts = new Hono<{ Variables: Posts }>();

export async function handleSinglePost(c: Context) {
  const postSlug: string = c.req.param("slug");
  const postService: PostService = c.get('postService');

  try {
    const post = postService.getPost(postSlug);

    if (process.env.DEPLOY_MODE !== "development") {
      if (post.meta.draft) {
        throw Error("Unable to find post.");
      }
    }

    const meta: SiteMeta = {
      title: post.meta.title,
      description: post.meta.description,
      tags: post.meta.tags,
      author: "Dave Smith-Hayes"
    };

    return c.render(<PostPage post={post} />, { meta });
  } catch (e) {
    const description: string = "Page does not exist.";
    console.error(description);
    console.error(e);

    c.status(404);
    const meta: SiteMeta = { description };
    return c.render(<FourOhFour />, { meta });
  }
}

posts.get('/:slug', handleSinglePost);
export default posts;

