import { Hono } from 'hono';
import { jsxRenderer } from 'hono/jsx-renderer';
import { serveStatic } from 'hono/bun';
import { Page } from '@blog/templates/Page';
import home from '@blog/handlers/home';
import posts from '@blog/handlers/posts';
import feed from '@blog/handlers/feed';
import type { SiteMeta } from '@blog/models/SiteMeta';
import { postFileMiddleware } from '@blog/middleware/post-service';
import { logger } from 'hono/logger';

declare module 'hono' {
  interface ContextRenderer {
    (content: string|Promise<string>, props: { meta: SiteMeta }): Response;
  }
}

const app = new Hono();

app.use(postFileMiddleware);
app.use(logger());

// JSX Rendering for Templates
app.get(
  '*',
  jsxRenderer(
    ({ children, meta }) => <Page meta={meta}>{children}</Page>,
    { docType: true }
  )
);

app.use('/static/*', serveStatic({
  root: '/src/assets',
  onNotFound: (path, c) => {
    console.error(`${path} is not found; Trying to access ${c.req.path}`);
  }
}));

app.route('/', home);
app.route('/feed', feed)
app.route('/posts', posts);

console.log("Starting the blog");
export default {
  port: process.env.APP_PORT || 3000,
  fetch: app.fetch
}

