import { Feed, FeedOptions } from 'feed';
import { getSiteConfig } from '@blog/services/config';
import { PostService } from '@blog/services/post-file';
import type { Post } from '@blog/models/post';

const updatedDate = new Date();

export async function getFeed(postService: PostService): Feed {
  const config = await getSiteConfig();
  const feed = new Feed({
    title: config.title,
    description: config.site.description,
    link: config.site.link,
    language: config.site.language,
    copyright: config.site.copyright,
    generator: "dsh feed gen",
    updated: updatedDate,
    author: {
      name: "Dave Smith-Hayes",
      email: "me@davesmithhayes.com",
      link: "davesmithhayes.com"
    }
  });

  const fullSlug = (slug) => config.site.url + "/posts/" + slug;

  postService.getPublishedPosts().map((post: Post) => {
    feed.addItem({
      title: post.meta.title,
      id: fullSlug(post.meta.slug),
      link: fullSlug(post.meta.slug),
      description: post.meta.description,
      content: post.html,
      author: {
        name: config.author,
        email: config.email
      }
    });
  });

  return feed;
};

