import { SiteMeta } from '@blog/models/SiteMeta';

export function Description({ meta }: { meta: SiteMeta }) {
  if (!meta?.description) {
    return (<></>);
  }

  return (<meta name="description" content={meta.description} />);
}

export function Tags({ meta }: { meta: SiteMeta }) {
  if (!meta?.tags) {
    return (<></>);
  }

  return (<meta name="keywords" content={meta.tags.join(', ')} />);
}

export function Author({ meta }: { meta: SiteMeta }) {
  if (!meta?.author) {
    return (<></>);
  }

  return (<meta name="author" content={meta.author} />);
}

export function ViewPort({ meta }: { meta: SiteMeta }) {
  if (!meta?.viewport) {
    return (
      <meta 
        name="viewport" 
        content="width=device-width, initial-scale=1.0" />
    );
  }

  return (<meta name="viewport" content={meta.viewport} />); 
}

export function MeRel({ meta }: { meta: SiteMeta }) {
  if (!meta?.mastodonUrl) {
    return (
      <></>
    );
  }

  return (<link rel="me" href={meta.mastodonUrl} />);
}

export function MetaTags({ meta }: { meta: SiteMeta }) {
  return (
    <>
      <Description meta={meta} />
      <Tags meta={meta} />
      <Author meta={meta} />
      <ViewPort meta={meta} />
      <MeRel meta={meta} />
    </>
  )
}
